@@ -2,4 +2,5 @@
 #define MFBSTIPPLEFILLAREA xf1bppStippleWhiteArea
 #define OPEQ |=
 #define EQWHOLEWORD =~0
-#include "../../../../mfb/mfbpntarea.c"
+#include "mfbmap.h"
+#include "../../../mfb/mfbpntarea.c"
